var inputFileName  = "DR14Q_v3_1-Z.csv";
var outputFileName = "DR14Q_v3_1-Z-HR-histogram2.csv";
var fso = new ActiveXObject("Scripting.FileSystemObject");
var DH = 13.77, DH3 = DH*DH*DH;
//
Number.prototype.finitize = function(value)
{	return isFinite(this) ? this : (arguments.length > 0 ? Number(value).finitize() : 0);
};
Number.prototype.crop = function(argMin,argMax)
{	if (arguments.length > 0 && this < Number(argMin) ) return NaN;
	if (arguments.length > 1 && this > Number(argMax) ) return NaN;
	return this;
};
Number.prototype.format = function()
{	return isFinite(this) ? this.toFixed(5) : "";
};
function zetaFromBeta(beta)			// relativistic Doppler effect
{	return Math.sqrt((1+beta)/(1-beta));
};
function betaFromZeta(zeta)			// inverse relativistic Doppler effect
{	var zeta2 = zeta*zeta;
	return (zeta2-1)/(zeta2+1);
};
function rhoFromBeta(beta)			// Hubble's Law for proper distance
{	return beta;
};
function tauFromBeta(beta)			// Hubble's Law for lookback time
{	return beta/(1+beta);
};
function psiFromRhoE(rho,dRho)			// dimensionless Euclidean shell volume
{	return dRho*4*Math.PI*rho*rho;
};
function psiFromRho3(rho,dRho)			// dimensionless 3-spherical shell volume
{	var s = Math.sin(Math.PI*rho);
	return dRho*4*s*s/Math.PI;
};
function doit()
{	WScript.StdOut.WriteLine("Creating "+outputFileName);
	var outStream = fso.CreateTextFile(outputFileName,true);
	WScript.StdOut.WriteLine("Reading from "+inputFileName);
	var inStream = fso.OpenTextFile(inputFileName,1);
	for (var betaResolution = 100, histogram = []; histogram.length < betaResolution; histogram.push(0) );
	for (var n = 0; !inStream.AtEndOfStream; n++)
	{	if (n%10000 == 0) WScript.StdOut.WriteLine("".concat("so far: n = ",n));
		var inLine = inStream.ReadLine();
		var z = Number(inLine);
		if (isFinite(z) && z > -1)
		{	var beta = betaFromZeta(z+1);
			histogram[Math.floor(beta*betaResolution)]++;
		};
	};
	WScript.StdOut.WriteLine("".concat("Completed reading: n = ",n));
	inStream .Close();
	WScript.StdOut.WriteLine("Scaling results...");
	var dRho  = 1/betaResolution;
// determine averages:
	for (var totCount = 0, totXiE = 0, totXi3 = 0, ih = 0; ih < histogram.length; ih++)
	{	var rho   = rhoFromBeta(ih/betaResolution);
		var count = histogram[ih];
		totCount += count;
		totXiE   += (count/(psiFromRhoE(rho,dRho)*DH3)).finitize();
		totXi3   += (count/(psiFromRho3(rho,dRho)*DH3)).finitize();
	};
	var avgCount = Math.round(totCount/histogram.length);
	var avgXiE   = Math.round(totXiE  /histogram.length);
	var avgXi3   = Math.round(totXi3  /histogram.length);
// search rho values where xi equals avgXi:
	for (var rhoE1 = NaN, rho31 = NaN, xiE = NaN, xi3 = NaN, ih = 0; ih < histogram.length; ih++)
	{	var rho    = rhoFromBeta(ih/betaResolution);
		var count  = histogram[ih];
		var prvXiE = xiE, xiE = (count/(psiFromRhoE(rho,dRho)*DH3)).finitize();
		var prvxi3 = xi3, xi3 = (count/(psiFromRho3(rho,dRho)*DH3)).finitize();
		if (isNaN(rhoE1) && prvXiE <= avgXiE && avgXiE <= xiE) rhoE1 = rho-dRho + dRho*(avgXiE-prvXiE)/(xiE-prvXiE);
		if (isNaN(rho31) && prvxi3 <= avgXi3 && avgXi3 <= xi3) rho31 = rho-dRho + dRho*(avgXi3-prvxi3)/(xi3-prvxi3);
		if (isFinite(rhoE1) && isFinite(rho31)) break;
	};
	var volE1 = Math.pow(histogram.length-rhoE1*betaResolution,3);	// scaled volume of universe
	var vol31 = Math.pow(histogram.length-rho31*betaResolution,3);
	WScript.StdOut.WriteLine("Writing to "+outputFileName);
	outStream.WriteLine("".concat('"rho"'
		,',"count/'		,avgCount	,' at distance rho, dRho=DH/100, DH='+DH.toFixed(2)+'Gly"'
		,',"Euclidean count/'	,avgXiE		,' per Gly3 at rho"'
		,',"normalised Euclidean 1/vol at age=1-rho"'
		,',"LN of normalised Euclidean density to 1/vol ratio"'
		,',"3-spherical count/'	,avgXi3		,' per Gly3 at rho"'
		,',"normalised 3-spherical 1/vol at age=1-rho"'
		,',"LN of normalised 3-spherical density to 1/vol ratio"'
		));
	for (var ih = 0; ih < histogram.length; ih++)
	{	var beta  = ih/betaResolution;
		var rho   = rhoFromBeta(beta);
		var z     = zetaFromBeta(beta).finitize()-1;
		var tau   = tauFromBeta(beta);
		var count = histogram[ih];
		var xiE   = (count/(psiFromRhoE(rho,dRho)*DH3)).finitize()/avgXiE;
		var xi3   = (count/(psiFromRho3(rho,dRho)*DH3)).finitize()/avgXi3;
		var vol   = Math.pow(histogram.length-ih,3); 
		var rcpVolE  = (volE1/vol).finitize();
		var rcpVol3  = (vol31/vol).finitize();
		count /= avgCount;
		outStream.WriteLine(
			[rho.toFixed(2)	,count.toFixed(5)
			,xiE.toFixed(5)	,rcpVolE.toFixed(5)	,Math.log(xiE/rcpVolE).crop(-5,2).format()
			,xi3.toFixed(5)	,rcpVol3.toFixed(5)	,Math.log(xi3/rcpVol3).crop(-5,2).format()
			]);
	};
	outStream.Close();
	WScript.StdOut.WriteLine("Done");
};
doit();
